@@ -20,7 +20,7 @@
 # USA
 
 import sys
-import gobject
+from gi import _gobject
 from gi.repository import GObject
 from ..overrides import override
 from ..importer import modules
@@ -55,6 +55,9 @@ class Widget(Gtk.Widget):
         if success:
             return (dest_x, dest_y,)
 
+    def render_icon(self, stock_id, size, detail=None):
+        return super(Widget, self).render_icon(stock_id, size, detail)
+
 Widget = override(Widget)
 __all__.append('Widget')
 
@@ -152,7 +155,10 @@ class ActionGroup(Gtk.ActionGroup):
         def _process_action(name, stock_id=None, label=None, accelerator=None, tooltip=None, callback=None):
             action = Action(name, label, tooltip, stock_id)
             if callback is not None:
-                action.connect('activate', callback, user_data)
+                if user_data is None:
+                    action.connect('activate', callback)
+                else:
+                    action.connect('activate', callback, user_data)
 
             self.add_action_with_accel(action, accelerator)
 
@@ -198,7 +204,10 @@ class ActionGroup(Gtk.ActionGroup):
             action = Gtk.ToggleAction(name, label, tooltip, stock_id)
             action.set_active(is_active)
             if callback is not None:
-                action.connect('activate', callback, user_data)
+                if user_data is None:
+                    action.connect('activate', callback)
+                else:
+                    action.connect('activate', callback, user_data)
 
             self.add_action_with_accel(action, accelerator)
 
@@ -266,7 +275,10 @@ class ActionGroup(Gtk.ActionGroup):
                 first_action = action
 
         if first_action is not None and on_change is not None:
-            first_action.connect('changed', on_change, user_data)
+            if user_data is None:
+                action.connect('changed', on_change)
+            else:
+                action.connect('changed', on_change, user_data)
 
 ActionGroup = override(ActionGroup)
 __all__.append('ActionGroup')
@@ -296,6 +308,32 @@ class ComboBox(Gtk.ComboBox, Container):
 ComboBox = override(ComboBox)
 __all__.append('ComboBox')
 
+
+class Box(Gtk.Box):
+    def __init__(self, homogeneous=False, spacing=0, **kwds):
+        super(Box, self).__init__(**kwds)
+        self.set_homogeneous(homogeneous)
+        self.set_spacing(spacing)
+
+Box = override(Box)
+__all__.append('Box')
+
+class SizeGroup(Gtk.SizeGroup):
+    def __init__(self, mode=Gtk.SizeGroupMode.VERTICAL):
+        super(SizeGroup, self).__init__(mode=mode)
+
+SizeGroup = override(SizeGroup)
+__all__.append('SizeGroup')
+
+
+class MenuItem(Gtk.MenuItem):
+    def __init__(self, label=None):
+        super(MenuItem, self).__init__(label=label)
+
+MenuItem = override(MenuItem)
+__all__.append('MenuItem')
+
+
 class Builder(Gtk.Builder):
 
     def connect_signals(self, obj_or_map):
@@ -377,11 +415,15 @@ class Dialog(Gtk.Dialog, Container):
             if flags & Gtk.DialogFlags.NO_SEPARATOR:
                 self.set_has_separator(False)
         except AttributeError:
-            pass
+            import warnings
+            warnings.warn("Gtk.DialogFlags.NO_SEPARATOR has been depricated since Gtk+-3.0", DeprecationWarning)
 
         if buttons is not None:
             self.add_buttons(*buttons)
 
+    action_area = property(lambda dialog: dialog.get_action_area())
+    vbox = property(lambda dialog: dialog.get_content_area())
+
     def add_buttons(self, *args):
         """
         The add_buttons() method adds several buttons to the Gtk.Dialog using
@@ -421,10 +463,11 @@ class MessageDialog(Gtk.MessageDialog, Dialog):
         if message_format != None:
             kwds['text'] = message_format
 
+        # type keyword is used for backwards compat with PyGTK
         if 'type' in kwds:
             import warnings
             warnings.warn("The use of the keyword type as a parameter of the Gtk.MessageDialog constructor has been depricated. Please use message_type instead.", DeprecationWarning)
-            message_type = kwds['type']
+            message_type = kwds.pop('type')
 
         Gtk.MessageDialog.__init__(self,
                                    _buttons_property=buttons,
@@ -582,11 +625,10 @@ class TextBuffer(Gtk.TextBuffer):
     def set_text(self, text, length=-1):
         Gtk.TextBuffer.set_text(self, text, length)
 
-    def insert(self, iter, text):
+    def insert(self, iter, text, length=-1):
         if not isinstance(text , _basestring):
             raise TypeError('text must be a string, not %s' % type(text))
 
-        length = len(text)
         Gtk.TextBuffer.insert(self, iter, text, length)
 
     def insert_with_tags(self, iter, text, *tags):
@@ -615,11 +657,10 @@ class TextBuffer(Gtk.TextBuffer):
 
         self.insert_with_tags(iter, text, *tag_objs)
 
-    def insert_at_cursor(self, text):
+    def insert_at_cursor(self, text, length=-1):
         if not isinstance(text , _basestring):
             raise TypeError('text must be a string, not %s' % type(text))
 
-        length = len(text)
         Gtk.TextBuffer.insert_at_cursor(self, text, length)
 
     def get_selection_bounds(self):
@@ -691,6 +732,10 @@ class TreeModel(Gtk.TreeModel):
                 raise IndexError("could not find tree path '%s'" % key)
             return TreeModelRow(self, aiter)
 
+    def __setitem__(self, key, value):
+        row = self[key]
+        self.set_row(row.iter, value)
+
     def __iter__(self):
         return TreeModelRowIter(self, self.get_iter_first())
 
@@ -738,6 +783,8 @@ class TreeModel(Gtk.TreeModel):
     def set_row(self, treeiter, row):
         # TODO: Accept a dictionary for row
         # model.append(None,{COLUMN_ICON: icon, COLUMN_NAME: name})
+        if isinstance(row, str):
+            raise TypeError('Expected a list or tuple, but got str')
 
         n_columns = self.get_n_columns()
         if len(row) != n_columns:
@@ -750,6 +797,13 @@ class TreeModel(Gtk.TreeModel):
 
             self.set_value(treeiter, i, value)
 
+    def _decode_value(self, treeiter, column, value):
+        type_ = self.get_column_type(column)
+        if type_ == GObject.TYPE_STRING and sys.version_info < (3, 0):
+            value = value.decode('UTF-8')
+
+        return value
+
     def _convert_value(self, treeiter, column, value):
             if value is None:
                 return
@@ -796,10 +850,7 @@ class TreeModel(Gtk.TreeModel):
                 # so we don't have to do it here
                 value_container = GObject.Value()
                 value_container.init(type_)
-                if type_ == GObject.TYPE_PYOBJECT:
-                    value_container.set_boxed(value)
-                    value = value_container
-                elif type_ == GObject.TYPE_CHAR:
+                if type_ == GObject.TYPE_CHAR:
                     value_container.set_char(value)
                     value = value_container
                 elif type_ == GObject.TYPE_UCHAR:
@@ -827,6 +878,9 @@ class TreeModel(Gtk.TreeModel):
                 elif type_ == GObject.TYPE_UINT64:
                     value_container.set_uint64(value)
                     value = value_container
+                elif type_ == GObject.TYPE_PYOBJECT:
+                    value_container.set_boxed(value)
+                    value = value_container
 
             return value
 
@@ -845,6 +899,9 @@ class TreeModel(Gtk.TreeModel):
 
         return tuple(values)
 
+    def filter_new(self, root=None):
+        return super(TreeModel, self).filter_new(root)
+
 TreeModel = override(TreeModel)
 __all__.append('TreeModel')
 
@@ -903,10 +960,47 @@ class ListStore(Gtk.ListStore, TreeModel, TreeSortable):
 
         return treeiter
 
+    def prepend(self, row=None):
+        treeiter = Gtk.ListStore.prepend(self)
+
+        if row is not None:
+            self.set_row(treeiter, row)
+
+        return treeiter
+
     def set_value(self, treeiter, column, value):
         value = self._convert_value(treeiter, column, value)
         Gtk.ListStore.set_value(self, treeiter, column, value)
 
+    def get_value(self, treeiter, column):
+        value = Gtk.ListStore.get_value(self, treeiter, column)
+        return self._decode_value(treeiter, column, value)
+
+    def set(self, treeiter, *args):
+
+        def _set_lists(columns, values):
+            if len(columns) != len(values):
+                raise TypeError('The number of columns do not match the number of values')
+            for col_num, val in zip(columns, values):
+                if not isinstance(col_num, int):
+                    raise TypeError('TypeError: Expected integer argument for column.')
+                self.set_value(treeiter, col_num, val)
+
+        if args:
+            if isinstance(args[0], int):
+                columns = args[::2]
+                values = args[1::2]
+                _set_lists(columns, values)
+            elif isinstance(args[0], (tuple, list)):
+                if len(args) != 2:
+                    raise TypeError('Too many arguments');
+                _set_lists(args[0], args[1])
+            elif isinstance(args[0], dict):
+                columns = args[0].keys()
+                values = args[0].values()
+                _set_lists(columns, values)
+            else:
+                raise TypeError('Argument list must be in the form of (column, value, ...), ((columns,...), (values, ...)) or {column: value}.  No -1 termination is needed.')
 
 ListStore = override(ListStore)
 __all__.append('ListStore')
@@ -954,6 +1048,12 @@ class TreeModelRow(object):
             elif key < 0:
                 key = self._convert_negative_index(key)
             return self.model.get_value(self.iter, key)
+        elif isinstance(key, slice):
+            start, stop, step = key.indices(self.model.get_n_columns())
+            alist = []
+            for i in range(start, stop, step):
+                alist.append(self.model.get_value(self.iter, i))
+            return alist
         else:
             raise TypeError("indices must be integers, not %s" % type(key).__name__)
 
@@ -963,9 +1063,19 @@ class TreeModelRow(object):
                 raise IndexError("column index is out of bounds: %d" % key)
             elif key < 0:
                 key = self._convert_negative_index(key)
-            return self.model.set_value(self.iter, key, value)
+            self.model.set_value(self.iter, key, value)
+        elif isinstance(key, slice):
+            start, stop, step = key.indices(self.model.get_n_columns())
+            indexList = range(start, stop, step)
+            if len(indexList) != len(value):
+                raise ValueError(
+                    "attempt to assign sequence of size %d to slice of size %d"
+                        % (len(value), len(indexList)))
+
+            for i,v in enumerate(indexList):
+                self.model.set_value(self.iter, v, value[i])
         else:
-            raise TypeError("indices must be integers, not %s" % type(key).__name__)
+            raise TypeError("index must be an integer or slice, not %s" % type(key).__name__)
 
     def _convert_negative_index(self, index):
         new_index = self.model.get_n_columns() + index
@@ -1081,12 +1191,46 @@ class TreeStore(Gtk.TreeStore, TreeModel, TreeSortable):
         value = self._convert_value(treeiter, column, value)
         Gtk.TreeStore.set_value(self, treeiter, column, value)
 
+    def get_value(self, treeiter, column):
+        value = Gtk.TreeStore.get_value(self, treeiter, column)
+        return self._decode_value(treeiter, column, value)
+
+    def set(self, treeiter, *args):
+
+        def _set_lists(columns, values):
+            if len(columns) != len(values):
+                raise TypeError('The number of columns do not match the number of values')
+            for col_num, val in zip(columns, values):
+                if not isinstance(col_num, int):
+                    raise TypeError('TypeError: Expected integer argument for column.')
+                self.set_value(treeiter, col_num, val)
+
+        if args:
+            if isinstance(args[0], int):
+                columns = args[::2]
+                values = args[1::2]
+                _set_lists(columns, values)
+            elif isinstance(args[0], (tuple, list)):
+                if len(args) != 2:
+                    raise TypeError('Too many arguments');
+                _set_lists(args[0], args[1])
+            elif isinstance(args[0], dict):
+                columns = args[0].keys()
+                values = args[0].values()
+                _set_lists(columns, values)
+            else:
+                raise TypeError('Argument list must be in the form of (column, value, ...), ((columns,...), (values, ...)) or {column: value}.  No -1 termination is needed.')
 
 TreeStore = override(TreeStore)
 __all__.append('TreeStore')
 
 class TreeView(Gtk.TreeView, Container):
 
+    def __init__(self, model=None):
+        Gtk.TreeView.__init__(self)
+        if model:
+            self.set_model(model)
+
     def get_path_at_pos(self, x, y):
         success, path, column, cell_x, cell_y = super(TreeView, self).get_path_at_pos(x, y)
         if success:
