@@ -32,6 +32,62 @@
 #include "pygi-marshal-cleanup.h"
 #include "pygi-marshal-from-py.h"
 
+/*
+ * _is_union_member - check to see if the py_arg is actually a member of the
+ * expected C union
+ */
+static gboolean
+_is_union_member (PyGIInterfaceCache *iface_cache, PyObject *py_arg) {
+    gint i;
+    gint n_fields;
+    GIUnionInfo *union_info;
+    GIInfoType info_type;
+    gboolean is_member = FALSE;
+
+    info_type = g_base_info_get_type (iface_cache->interface_info);
+
+    if (info_type != GI_INFO_TYPE_UNION)
+        return FALSE;
+
+    union_info = (GIUnionInfo *) iface_cache->interface_info;
+    n_fields = g_union_info_get_n_fields (union_info);
+
+    for (i = 0; i < n_fields; i++) {
+        GIFieldInfo *field_info;
+        GITypeInfo *field_type_info;
+
+        field_info = g_union_info_get_field (union_info, i);
+        field_type_info = g_field_info_get_type (field_info);
+
+        /* we can only check if the members are interfaces */
+        if (g_type_info_get_tag (field_type_info) == GI_TYPE_TAG_INTERFACE) {
+            PyObject *py_type;
+            GIInterfaceInfo *field_iface_info =
+                g_type_info_get_interface (field_type_info);
+
+            py_type = _pygi_type_import_by_gi_info (
+                (GIBaseInfo *) field_iface_info);
+
+            if (py_type != NULL && PyObject_IsInstance (py_arg, py_type)) {
+                is_member = TRUE;
+                break;
+            }
+
+            Py_XDECREF (py_type);
+            g_base_info_unref ( ( GIBaseInfo *) field_iface_info);
+
+        }
+
+        g_base_info_unref ( ( GIBaseInfo *) field_type_info);
+        g_base_info_unref ( ( GIBaseInfo *) field_info);
+
+        if (is_member)
+            break;
+    }
+
+    return is_member;
+}
+
 gboolean
 _pygi_marshal_from_py_void (PyGIInvokeState   *state,
                             PyGICallableCache *callable_cache,
@@ -738,7 +794,11 @@ _pygi_marshal_from_py_array (PyGIInvokeState   *state,
     if (sequence_cache->item_cache->type_tag == GI_TYPE_TAG_UINT8 &&
         PYGLIB_PyBytes_Check (py_arg)) {
         memcpy(array_->data, PYGLIB_PyBytes_AsString (py_arg), length);
-
+        if (sequence_cache->is_zero_terminated) {
+            /* If array_ has been created with zero_termination, space for the
+             * terminator is properly allocated, so we're not off-by-one here. */
+            array_->data[length] = '\0';
+        }
         goto array_success;
     }
 
@@ -775,8 +835,14 @@ _pygi_marshal_from_py_array (PyGIInvokeState   *state,
                     PyGIMarshalCleanupFunc from_py_cleanup = item_arg_cache->from_py_cleanup;
                     gboolean is_boxed = g_type_is_a (item_iface_cache->g_type, G_TYPE_BOXED);
                     gboolean is_gvalue = item_iface_cache->g_type == G_TYPE_VALUE;
-
-                    if (!is_boxed || is_gvalue) {
+                    gboolean is_gvariant = item_iface_cache->g_type == G_TYPE_VARIANT;
+                    
+                    if (is_gvariant) {
+                        /* Item size will always be that of a pointer,
+                         * since GVariants are opaque hence always passed by ref */
+                        g_assert (item_size == sizeof (item.v_pointer));
+                        g_array_insert_val (array_, i, item.v_pointer);
+                    } else if (!is_boxed || is_gvalue) {
                         memcpy (array_->data + (i * item_size), item.v_pointer, item_size);
                         if (from_py_cleanup)
                             from_py_cleanup (state, item_arg_cache, item.v_pointer, TRUE);
@@ -1325,10 +1391,15 @@ _pygi_marshal_from_py_interface_struct (PyGIInvokeState   *state,
 
         return (success == Py_None);
     } else if (!PyObject_IsInstance (py_arg, iface_cache->py_type)) {
-        PyErr_Format (PyExc_TypeError, "Expected %s, but got %s",
-                      iface_cache->type_name,
-                      iface_cache->py_type->ob_type->tp_name);
-        return FALSE;
+        /* first check to see if this is a member of the expected union */
+        if (!_is_union_member (iface_cache, py_arg)) {
+            if (!PyErr_Occurred())
+                PyErr_Format (PyExc_TypeError, "Expected %s, but got %s",
+                              iface_cache->type_name,
+                              iface_cache->py_type->ob_type->tp_name);
+
+            return FALSE;
+        }
     }
 
     if (g_type_is_a (iface_cache->g_type, G_TYPE_BOXED)) {
@@ -1417,59 +1488,13 @@ gboolean _pygi_marshal_from_py_interface_instance (PyGIInvokeState   *state,
 
             if (!PyObject_IsInstance (py_arg, iface_cache->py_type)) {
                 /* wait, we might be a member of a union so manually check */
-                if (info_type == GI_INFO_TYPE_UNION) {
-                    gint i;
-                    gint n_fields;
-                    gboolean is_member = FALSE;
-                    GIUnionInfo *union_info = (GIUnionInfo *) iface_cache->interface_info;
-
-                    n_fields = g_union_info_get_n_fields (union_info);
-
-                    for (i = 0; i < n_fields; i++) {
-                        GIFieldInfo *field_info;
-                        GITypeInfo *field_type_info;
-
-                        field_info = g_union_info_get_field (union_info, i);
-                        field_type_info = g_field_info_get_type (field_info);
-
-                        /* we can only check if the members are interfaces */
-                        if (g_type_info_get_tag (field_type_info) ==
-                            GI_TYPE_TAG_INTERFACE) {
-                            PyObject *py_type;
-                            GIInterfaceInfo *field_iface_info =
-                                g_type_info_get_interface(field_type_info);
-
-                            py_type = _pygi_type_import_by_gi_info (
-                                (GIBaseInfo *) field_iface_info);
-
-                            if (PyObject_IsInstance (py_arg, py_type)) {
-                                is_member = TRUE;
-                                break;
-                            }
-
-                            Py_XDECREF (py_type);
-                            g_base_info_unref ( ( GIBaseInfo *) field_iface_info);
-
-                        }
-
-                        g_base_info_unref ( ( GIBaseInfo *) field_type_info);
-                        g_base_info_unref ( ( GIBaseInfo *) field_info);
-
-                        if (is_member)
-                            break;
-                    }
-
-                    if (!is_member) {
+                if (!_is_union_member (iface_cache, py_arg)) {
+                    if (!PyErr_Occurred())
                         PyErr_Format (PyExc_TypeError,
                                       "Expected a %s, but got %s",
                                       iface_cache->type_name,
                                       py_arg->ob_type->tp_name);
-                        return FALSE;
-                    }
-                } else {
-                    PyErr_Format (PyExc_TypeError, "Expected a %s, but got %s",
-                                  iface_cache->type_name,
-                                  py_arg->ob_type->tp_name);
+
                     return FALSE;
                 }
             }
