@@ -350,6 +350,30 @@ class TestCallbacks(unittest.TestCase):
         # make sure this doesn't assert or crash
         Everything.test_simple_callback(None)
 
+class TestClosures(unittest.TestCase):
+    def test_int_arg(self):
+        def callback(num):
+            self.called = True
+            return num+1
+
+        self.called = False
+        result = Everything.test_closure_one_arg(callback, 42)
+        self.assertTrue(self.called)
+        self.assertEqual(result, 43)
+
+    # https://bugzilla.gnome.org/show_bug.cgi?id=656554
+    @unittest.expectedFailure
+    def test_variant(self):
+        def callback(variant):
+            self.assertEqual(variant.get_type_string(), 'i')
+            self.called = True
+            return GLib.Variant('i', variant.get_int32() + 1)
+
+        self.called = False
+        result = Everything.test_closure_variant(callback, GLib.Variant('i', 42))
+        self.assertTrue(self.called)
+        self.assertEqual(result.get_type_string(), 'i')
+        self.assertEqual(result.get_int32(), 43)
 
 class TestProperties(unittest.TestCase):
 
@@ -483,3 +507,22 @@ class TestAdvancedInterfaces(unittest.TestCase):
         self.assertEquals(ret[0], 51);
         self.assertEquals(ret[1], 61);
         self.assertEquals(ret[2], 32);
+
+    def test_obj_skip_return_val_no_out(self):
+        obj = Everything.TestObj();
+        # raises an error for 0, succeeds for any other value
+        self.assertRaises(GLib.GError, obj.skip_return_val_no_out, 0)
+
+        ret = obj.skip_return_val_no_out(1)
+        self.assertEquals(ret, None)
+
+class TestSignals(unittest.TestCase):
+    def test_object_param_signal(self):
+        obj = Everything.TestObj();
+
+        def callback (obj_param):
+            self.assertEquals(obj_param.props.int, 3)
+            self.assertEquals(obj_param.__grefcount__, 2)
+
+        obj.connect('sig-with-obj', callback)
+        obj.emit_sig_with_obj()
